% kf_dfan.m - filtro de Kalman filter para el ventilador de flujo guiado

%% Variables globales para comunicacion con el mdoo de simulacion
global pvtol_K pvtol_L pvtol_C pvtol_Rv pvtol_Rw pvtol_F;

%%
%% Dinamica del ventilador de flujo guiado%%
%% Esta es la dinamica del ventilador de flujo guiado, escrito en la forma
%% de espacio de estados.
%%

% Parametros del sistema
J = 0.0475;			% inertia around pitch axis
m = 1.5;			% mass of fan
r = 0.25;			% distance to flaps
g = 10;				% gravitational constant
d = 0.2;	 		% damping factor (estimated)

% Matrices del sistema (planta completa: 2 entradas, 2 salidas)
A = [  0     0     0     1     0     0;
       0     0     0     0     1     0;
       0     0     0     0     0     1;
       0     0    -g   -d/m    0     0;
       0     0     0     0   -d/m    0;
       0     0     0     0     0     0 ];
 
B = [  0     0;
       0     0;
       0     0;
      1/m    0;
       0    1/m;
      r/J    0    ];

C = [  1     0    0      0     0     0;
       0     1    0      0     0     0 ];

D = [  0     0;   0      0];

dfsys = ss(A, B, C, D);

%%
%% Diseno del control
%%
%% Usamos control LQR para elegir ganancias de realimentacion de estados
%%

K = lqr(A, B, eye(size(A)), 0.01*eye(size(B'*B)));
pvtol_K = K;

%%
%% Estimador #1
%% 

% Definir los disturbios y condicion inicial
pvtol_F = eye(6);
pvtol_Rw = diag([0.0001, 0.0001, 0.0001, 0.01, 0.04, 0.0001]);

x0 = [0.1 0.2 0 0 0 0];
R11 = 0.1; R22 = 0.1; R33 = 0.001;

% Definir matrices de ponderacion (L is computed but not used)
pvtol_C = [1 0 0 0 0 0; 0 1 0 0 0 0];
pvtol_Rv = diag([R11 R22]);
pvtol_L = lqe(A, pvtol_F, pvtol_C, pvtol_Rw, pvtol_Rv);

[t1, y1] = ode45(@pvtol, [0, 15], ...
  [x0 0*x0 reshape(x0'*x0, 1, 36)]);


subplot(321);
  plot(t1, y1(:,1), 'b-', t1, y1(:,2), 'g--');
  legend x y;
  xlabel('time');figure(1)
  ylabel('States (no \theta)');
  axis([0 15 -0.3 0.3]);

subplot(323); 
  plot(t1, y1(:,7) - y1(:,1), 'b-', ...
       t1, y1(:,8) - y1(:,2), 'g--', ...
       t1, y1(:,9) - y1(:,3), 'r-');
  legend xerr yerr terr;
  xlabel('time');
  ylabel('Errors (no \theta)');
  axis([0 15 -0.2 0.2]);

subplot(325);
  plot(t1, y1(:,13), 'b-', t1, y1(:,19), 'g--', t1, y1(:,25), 'r-');
  legend P11 P22 P33
  xlabel('time');
  ylabel('Covariance (w/ \theta)');
  axis([0 15 -0.2 0.2]);

%%
%% Estimador #2
%% 

% Ahora cambiar la salida y observar que pasa (L computed but not used)
pvtol_C = [1 0 0 0 0 0; 0 1 0 0 0 0; 0 0 1 0 0 0];
pvtol_Rv = diag([R11 R22 R33]);
pvtol_L = lqe(A, pvtol_F, pvtol_C, pvtol_Rw, pvtol_Rv);

[t2, y2] = ode45(@pvtol, [0, 15], ...
  [x0 0*x0 reshape(x0'*x0, 1, 36)]);


subplot(322);
  plot(t2, y2(:,1), 'b-', t2, y2(:,2), 'g--');
  legend x y;
  xlabel('time');
  ylabel('States (w/ \theta)');
  axis([0 15 -0.3 0.3]);

subplot(324);
  plot(t2, y2(:,7) - y2(:,1), 'b-', ...
       t2, y2(:,8) - y2(:,2), 'g--', ...
       t2, y2(:,9) - y2(:,3), 'r-');
  legend xerr yerr terr;
  xlabel('time');
  ylabel('Errors (w/ \theta)');
  axis([0 15 -0.2 0.2]);

subplot(326);
  plot(t2, y2(:,13), 'b-', t2, y2(:,19), 'g--', t2, y2(:,25), 'r-');
  legend P11 P22 P33
  xlabel('time');
  ylabel('Covariance (w/ \theta)');
  axis([0 15 -0.2 0.2]);

print -dpdf dfan_kf.pdf
