
%%
%% Dinamica del ventilador con flujo guiado
%%
%% Esta es la dinamica del ventilador de flujo guiado, escrito en la forma
%% de espacio de estados.
%%

% Parametros del sistema
J = 0.0475;			% inertia around pitch axis
m = 1.5;			% mass of fan
r = 0.25;			% distance to flaps
g = 10;				% gravitational constant
gamma = 0.51;			% counterweighted mass
d = 0.2;	 		% damping factor (estimated)
l = 0.05;			% offset of center of mass

% Modificar el amortiguamiento para mejores graficas
d = 1;

% Matrices del sistema (plant completa: 2 entradas, 2 salidas)
A = [  0     0     0     1     0     0;
       0     0     0     0     1     0;
       0     0     0     0     0     1;
       0     0  -gamma -d/m    0     0;
       0     0     0     0   -d/m    0;
       0     0 -m*g*l/J  0     0     0 ];
 
B = [  0     0;
       0     0;
       0     0;
      1/m    0;
       0    1/m;
      r/J    0    ];

C = [  1     0    0      0     0     0;
       0     1    0      0     0     0 ];

D = [  0     0;   0      0];

%% Extrayendo dinamica lateral
Ax = A([1 3 4 6], [1 3 4 6]);
Bx = B([1 3 4 6], 1);
Cx = C(1, [1 3 4 6]);
Dx = D(1, 1);

dfsys = ss(Ax, Bx, Cx, Dx);

%%
%% Diseno del control espacio de estados
%%
%% LQR
%%

K = lqr(Ax, Bx, eye(size(Ax)), 0.1*eye(size(Bx'*Bx)));

%%
%% Estimador
%% 
%% Ganancia del estimador por ubicacion de polos
%%

% Determinar polos deseados usando la configuracion de butterworth
[Af,Bf,Cf,Df]=butter(4,10,'s');
[numf, denf] = ss2tf(Af,Bf,Cf,Df);
eigs = pole(tf(numf,denf));

L1 = transpose( place(Ax', Cx', eigs) );

Lk = lqe(Ax, 0.1*eye(size(Ax)), Cx, 0.001*eye(size(Ax)), ...
	0.001*eye(size(Cx*Cx')), 0*eye(size(Ax*Cx')));

% Observar la respuesta del estimador a errores iniciales
% Cosntruir un sistema con salidas = estados estimados
est = ss(Ax - L1*Cx, Bx, eye(size(Ax)), 0);
figure(1); initial(est, [0.1; 0.01; 1/180*pi; 0]);

%%
%% Planta en lazo cerrado
%%
%% Usando la funcion "reg" para construir el regulador a partir 
%% del estimador y el controlador en espacio de estado.  
%% Especial atencion con el signo negativo
%%

dfctr1 = -tf(reg(dfsys, K, L1));
H1 = feedback(dfctr1*dfsys,1);

dfctrk = -tf(reg(dfsys, K, Lk));
Hk = feedback(dfctrk*dfsys,1);

%%
%% Analisis de la estructura del controlador
%%
margintf=tf(-1);		% for seeing gain/phase margins
figure(2); 
  W = logspace(-2, 3, 500);
  bode(dfsys, dfctr1, dfctr1*dfsys, margintf, W);
  legend('P', 'C', 'P*C', 'Location', 'NorthWest');

% Graficar la respuesta al escalon para un cambio tipo escalon en la salida
% deseada
% 
figure(3);
  subplot(211);
  step(feedback(dfctr1*dfsys, 1), 12);

  subplot(212);
  step(feedback(dfctrk*dfsys, 1), 12);
